@@ -1,9 +1,11 @@
 from .rcmod import *
 from .utils import *
+from .palettes import *
 from .linearmodels import *
 from .distributions import *
 from .timeseries import *
 from .miscplot import *
+from .axisgrid import *
 set()
 
-__version__ = "0.2.1"
+__version__ = "0.3.1"
