@@ -4,15 +4,21 @@ import colorsys
 import numpy as np
 from scipy import stats
 import pandas as pd
-import statsmodels.api as sm
 import matplotlib as mpl
 import matplotlib.pyplot as plt
-from six.moves import range
 import warnings
-import moss
 
-from seaborn.utils import (color_palette, husl_palette, blend_palette,
-                           desaturate, _kde_support)
+try:
+    import statsmodels.api as sm
+    _has_statsmodels = True
+except ImportError:
+    _has_statsmodels = False
+
+from .external.six.moves import range
+
+from .utils import set_hls_values, desaturate, percentiles, iqr, _kde_support
+from .palettes import color_palette, husl_palette, blend_palette
+from .axisgrid import JointGrid
 
 
 def _box_reshape(vals, groupby, names, order):
@@ -120,7 +126,7 @@ def _box_colors(vals, color):
 
 def boxplot(vals, groupby=None, names=None, join_rm=False, order=None,
             color=None, alpha=None, fliersize=3, linewidth=1.5, widths=.8,
-            ax=None, **kwargs):
+            label=None, ax=None, **kwargs):
     """Wrapper for matplotlib boxplot with better aesthetics and functionality.
 
     Parameters
@@ -196,21 +202,30 @@ def boxplot(vals, groupby=None, names=None, join_rm=False, order=None,
         fly.set_markeredgecolor(gray)
         fly.set_markersize(fliersize)
 
+    # This is a hack to get labels to work
+    # It's unclear whether this is actually broken in matplotlib or just not
+    # implemented, either way it's annoying.
+    if label is not None:
+        pos = kwargs.get("positions", [1])[0]
+        med = np.median(vals[0])
+        color = colors[0]
+        ax.add_patch(plt.Rectangle([pos, med], 0, 0, color=color, label=label))
+
     # Is this a vertical plot?
     vertical = kwargs.get("vert", True)
 
     # Draw the joined repeated measures
     if join_rm:
-        x, y = np.arange(1, len(np.transpose(vals)) + 1), np.transpose(vals)
+        x, y = np.arange(1, len(vals) + 1), vals
         if not vertical:
             x, y = y, x
         ax.plot(x, y, color=gray, alpha=2. / 3)
 
     # Label the axes and ticks
     if vertical:
-        ax.set_xticklabels(names)
+        ax.set_xticklabels(list(names))
     else:
-        ax.set_yticklabels(names)
+        ax.set_yticklabels(list(names))
         xlabel, ylabel = ylabel, xlabel
     if xlabel is not None:
         ax.set_xlabel(xlabel)
@@ -226,16 +241,9 @@ def boxplot(vals, groupby=None, names=None, join_rm=False, order=None,
     return ax
 
 
-def violin(*args, **kwargs):
-    """Deprecated old name for violinplot. Please update your code."""
-    warnings.warn("violin() is deprecated; please use violinplot()",
-                  UserWarning)
-    return violinplot(*args, **kwargs)
-
-
 def violinplot(vals, groupby=None, inner="box", color=None, positions=None,
-               names=None, order=None, kernel="gau", bw="scott", widths=.8,
-               alpha=None, join_rm=False, gridsize=100, cut=3, inner_kws=None,
+               names=None, order=None, bw="scott", widths=.8, alpha=None,
+               join_rm=False, gridsize=100, cut=3, inner_kws=None,
                ax=None, **kwargs):
 
     """Create a violin plot (a combination of boxplot and kernel density plot).
@@ -264,8 +272,6 @@ def violinplot(vals, groupby=None, inner="box", color=None, positions=None,
         If vals is a Pandas object with name information, you can control the
         order of the plot by providing the violin names in your preferred
         order.
-    kernel : {'gau' | 'cos' | 'biw' | 'epa' | 'tri' | 'triw' }
-        Code for shape of kernel to fit with.
     bw : {'scott' | 'silverman' | scalar}
         Name of reference method to determine kernel size, or size as a
         scalar.
@@ -304,11 +310,12 @@ def violinplot(vals, groupby=None, inner="box", color=None, positions=None,
     # Initialize the kwarg dict for the inner plot
     if inner_kws is None:
         inner_kws = {}
-    in_alpha = inner_kws.pop("alpha", .6 if inner == "points" else 1)
-    in_alpha *= 1 if alpha is None else alpha
-    in_color = inner_kws.pop("color", gray)
-    in_marker = inner_kws.pop("marker", ".")
-    in_lw = inner_kws.pop("lw", 1.5 if inner == "box" else .8)
+    inner_kws.setdefault("alpha", .6 if inner == "points" else 1)
+    inner_kws["alpha"] *= 1 if alpha is None else alpha
+    inner_kws.setdefault("color", gray)
+    inner_kws.setdefault("marker", "." if inner == "points" else "")
+    lw = inner_kws.pop("lw", 1.5 if inner == "box" else .8)
+    inner_kws.setdefault("linewidth", lw)
 
     # Find where the violins are going
     if positions is None:
@@ -327,49 +334,49 @@ def violinplot(vals, groupby=None, inner="box", color=None, positions=None,
 
         # Fit the KDE
         x = positions[i]
-        kde = sm.nonparametric.KDEUnivariate(a)
-        fft = kernel == "gau"
-        kde.fit(bw=bw, kernel=kernel, gridsize=gridsize, cut=cut, fft=fft)
-        y, dens = kde.support, kde.density
+        kde = stats.gaussian_kde(a, bw)
+        if isinstance(bw, str):
+            bw_name = "scotts" if bw == "scott" else bw
+            _bw = getattr(kde, "%s_factor" % bw_name)() * a.std(ddof=1)
+        else:
+            _bw = bw
+        y = _kde_support(a, _bw, gridsize, cut, (-np.inf, np.inf))
+        dens = kde.evaluate(y)
         scl = 1 / (dens.max() / (widths / 2))
         dens *= scl
 
         # Draw the violin
         ax.fill_betweenx(y, x - dens, x + dens, alpha=alpha, color=colors[i])
         if inner == "box":
-            for quant in moss.percentiles(a, [25, 75]):
+            for quant in percentiles(a, [25, 75]):
                 q_x = kde.evaluate(quant) * scl
                 q_x = [x - q_x, x + q_x]
-                ax.plot(q_x, [quant, quant], color=in_color,
-                        linestyle=":", linewidth=in_lw, **inner_kws)
+                ax.plot(q_x, [quant, quant], linestyle=":",  **inner_kws)
             med = np.median(a)
             m_x = kde.evaluate(med) * scl
             m_x = [x - m_x, x + m_x]
-            ax.plot(m_x, [med, med], color=in_color,
-                    linestyle="--", linewidth=in_lw, **inner_kws)
+            ax.plot(m_x, [med, med], linestyle="--", **inner_kws)
         elif inner == "stick":
             x_vals = kde.evaluate(a) * scl
             x_vals = [x - x_vals, x + x_vals]
-            ax.plot(x_vals, [a, a], color=in_color,
-                    linewidth=in_lw, alpha=in_alpha, **inner_kws)
+            ax.plot(x_vals, [a, a], linestyle="-", **inner_kws)
         elif inner == "points":
             x_vals = [x for _ in a]
-            ax.plot(x_vals, a, in_marker, color=in_color,
-                    alpha=in_alpha, mew=0, **inner_kws)
+            ax.plot(x_vals, a, mew=0, linestyle="", **inner_kws)
         for side in [-1, 1]:
             ax.plot((side * dens) + x, y, c=gray, lw=lw)
 
     # Draw the repeated measure bridges
     if join_rm:
         ax.plot(range(1, len(vals) + 1), vals,
-                color=in_color, alpha=2. / 3)
+                color=inner_kws["color"], alpha=2. / 3)
 
     # Add in semantic labels
     ax.set_xticks(positions)
     if names is not None:
         if len(vals) != len(names):
             raise ValueError("Length of names list must match nuber of bins")
-        ax.set_xticklabels(names)
+        ax.set_xticklabels(list(names))
     ax.set_xlim(positions[0] - .5, positions[-1] + .5)
 
     if xlabel is not None:
@@ -385,39 +392,43 @@ def _freedman_diaconis_bins(a):
     """Calculate number of hist bins using Freedman-Diaconis rule."""
     # From http://stats.stackexchange.com/questions/798/
     a = np.asarray(a)
-    h = 2 * moss.iqr(a) / (len(a) ** (1 / 3))
+    h = 2 * iqr(a) / (len(a) ** (1 / 3))
     return np.ceil((a.max() - a.min()) / h)
 
 
 def distplot(a, bins=None, hist=True, kde=True, rug=False, fit=None,
              hist_kws=None, kde_kws=None, rug_kws=None, fit_kws=None,
-             color=None, vertical=False, axlabel=None, ax=None):
+             color=None, vertical=False, axlabel=None, label=None, ax=None):
     """Flexibly plot a distribution of observations.
 
-    Parameter
+    Parameters
+    ----------
+
     a : (squeezable to) 1d array
         Observed data.
-    bins : argument for matplotlib hist(), or None
+    bins : argument for matplotlib hist(), or None, optional
         Specification of hist bins, or None to use Freedman-Diaconis rule.
-    hist : bool, default True
+    hist : bool, optional
         Whether to plot a (normed) histogram.
-    kde : bool, default True
+    kde : bool, optional
         Whether to plot a gaussian kernel density estimate.
-    rug : bool, default False
+    rug : bool, optional
         Whether to draw a rugplot on the support axis.
-    fit : random variable object
+    fit : random variable object, optional
         An object with `fit` method, returning a tuple that can be passed to a
         `pdf` method a positional arguments following an grid of values to
         evaluate the pdf on.
-    {hist, kde, rug, fit}_kws : dictionaries
+    {hist, kde, rug, fit}_kws : dictionaries, optional
         Keyword arguments for underlying plotting functions.
     color : matplotlib color, optional
         Color to plot everything but the fitted curve in.
-    vertical : bool, default False
+    vertical : bool, optional
         If True, oberved values are on y-axis.
-    axlabel : string, False, or None
+    axlabel : string, False, or None, optional
         Name for the support axis label. If None, will try to get it
         from a.namel if False, do not set a label.
+    label : string, optional
+        Legend label for the relevent component of the plot
     ax : matplotlib axis, optional
         if provided, plot on this axis
 
@@ -458,30 +469,47 @@ def distplot(a, bins=None, hist=True, kde=True, rug=False, fit=None,
         color = line.get_color()
         line.remove()
 
+    # Plug the label into the right kwarg dictionary
+    if label is not None:
+        if hist:
+            hist_kws["label"] = label
+        elif kde:
+            kde_kws["label"] = label
+        elif rug:
+            rug_kws["label"] = label
+        elif fit:
+            fit_kws["label"] = label
+
     if hist:
         if bins is None:
             bins = _freedman_diaconis_bins(a)
-        hist_alpha = hist_kws.pop("alpha", 0.4)
+        hist_kws.setdefault("alpha", 0.4)
         orientation = "horizontal" if vertical else "vertical"
         hist_color = hist_kws.pop("color", color)
-        ax.hist(a, bins, normed=True, color=hist_color, alpha=hist_alpha,
-                orientation=orientation, **hist_kws)
+        ax.hist(a, bins, normed=True, orientation=orientation,
+                color=hist_color, **hist_kws)
+        if hist_color != color:
+            hist_kws["color"] = hist_color
 
     if kde:
         kde_color = kde_kws.pop("color", color)
-        kdeplot(a, vertical=vertical, color=kde_color, ax=ax, **kde_kws)
+        kdeplot(a, vertical=vertical, ax=ax, color=kde_color, **kde_kws)
+        if kde_color != color:
+            kde_kws["color"] = kde_color
 
     if rug:
         rug_color = rug_kws.pop("color", color)
         axis = "y" if vertical else "x"
-        rugplot(a, axis=axis, color=rug_color, ax=ax, **rug_kws)
+        rugplot(a, axis=axis, ax=ax, color=rug_color, **rug_kws)
+        if rug_color != color:
+            rug_kws["color"] = rug_color
 
     if fit is not None:
         fit_color = fit_kws.pop("color", "#282828")
-        gridsize = fit_kws.pop("gridsize", 500)
+        gridsize = fit_kws.pop("gridsize", 200)
         cut = fit_kws.pop("cut", 3)
         clip = fit_kws.pop("clip", (-np.inf, np.inf))
-        bw = sm.nonparametric.bandwidths.bw_scott(a)
+        bw = stats.gaussian_kde(a).scotts_factor() * a.std(ddof=1)
         x = _kde_support(a, bw, gridsize, cut, clip)
         params = fit.fit(a)
         pdf = lambda x: fit.pdf(x, *params)
@@ -489,6 +517,8 @@ def distplot(a, bins=None, hist=True, kde=True, rug=False, fit=None,
         if vertical:
             x, y = y, x
         ax.plot(x, y, color=fit_color, **fit_kws)
+        if fit_color != "#282828":
+            fit_kws["color"] = fit_color
 
     if label_ax:
         if vertical:
@@ -500,7 +530,7 @@ def distplot(a, bins=None, hist=True, kde=True, rug=False, fit=None,
 
 
 def _univariate_kdeplot(data, shade, vertical, kernel, bw, gridsize, cut,
-                        clip, legend, ax, **kwargs):
+                        clip, legend, ax, cumulative=False, **kwargs):
     """Plot a univariate kernel density estimate on one of the axes."""
 
     # Sort out the clipping
@@ -508,16 +538,21 @@ def _univariate_kdeplot(data, shade, vertical, kernel, bw, gridsize, cut,
         clip = (-np.inf, np.inf)
 
     # Calculate the KDE
-    try:
+    if _has_statsmodels:
         # Prefer using statsmodels for kernel flexibility
         x, y = _statsmodels_univariate_kde(data, kernel, bw,
-                                           gridsize, cut, clip)
-    except ImportError:
+                                           gridsize, cut, clip,
+                                           cumulative=cumulative)
+    else:
         # Fall back to scipy if missing statsmodels
         if kernel != "gau":
             kernel = "gau"
             msg = "Kernel other than `gau` requires statsmodels."
             warnings.warn(msg, UserWarning)
+        if cumulative:
+            raise ImportError("Cumulative distributions are currently"
+                              "only implemented in statsmodels."
+                              "Please install statsmodels.")
         x, y = _scipy_univariate_kde(data, bw, gridsize, cut, clip)
 
     # Make sure the density is nonnegative
@@ -535,7 +570,7 @@ def _univariate_kdeplot(data, shade, vertical, kernel, bw, gridsize, cut,
         label = data.name
 
     # Decide if we're going to add a legend
-    legend = not label is None and legend
+    legend = label is not None and legend
     label = "_nolegend_" if label is None else label
 
     # Use the active color cycle to find the plot color
@@ -546,9 +581,12 @@ def _univariate_kdeplot(data, shade, vertical, kernel, bw, gridsize, cut,
 
     # Draw the KDE plot and, optionally, shade
     ax.plot(x, y, color=color, label=label, **kwargs)
-    alpha = kwargs.pop("alpha", 0.25)
+    alpha = kwargs.get("alpha", 0.25)
     if shade:
-        ax.fill_between(x, 1e-12, y, color=color, alpha=alpha)
+        if vertical:
+            ax.fill_betweenx(y, 1e-12, x, color=color, alpha=alpha)
+        else:
+            ax.fill_between(x, 1e-12, y, color=color, alpha=alpha)
 
     # Draw the legend here
     if legend:
@@ -557,13 +595,16 @@ def _univariate_kdeplot(data, shade, vertical, kernel, bw, gridsize, cut,
     return ax
 
 
-def _statsmodels_univariate_kde(data, kernel, bw, gridsize, cut, clip):
+def _statsmodels_univariate_kde(data, kernel, bw, gridsize, cut, clip,
+                                cumulative=False):
     """Compute a univariate kernel density estimate using statsmodels."""
-    from statsmodels import nonparametric
     fft = kernel == "gau"
-    kde = nonparametric.kde.KDEUnivariate(data)
+    kde = sm.nonparametric.KDEUnivariate(data)
     kde.fit(kernel, bw, fft, gridsize=gridsize, cut=cut, clip=clip)
-    grid, y = kde.support, kde.density
+    if cumulative:
+        grid, y = kde.support, kde.cdf
+    else:
+        grid, y = kde.support, kde.density
     return grid, y
 
 
@@ -589,21 +630,23 @@ def _bivariate_kdeplot(x, y, filled, kernel, bw, gridsize, cut, clip, axlabel,
         clip = [clip, clip]
 
     # Calculate the KDE
-    try:
+    if _has_statsmodels:
         xx, yy, z = _statsmodels_bivariate_kde(x, y, bw, gridsize, cut, clip)
-    except ImportError:
+    else:
         xx, yy, z = _scipy_bivariate_kde(x, y, bw, gridsize, cut, clip)
 
     # Plot the contours
     n_levels = kwargs.pop("n_levels", 10)
-    cmap = kwargs.pop("cmap", "BuGn" if filled else "BuGn_d")
+    cmap = kwargs.get("cmap", "BuGn" if filled else "BuGn_d")
     if isinstance(cmap, str):
         if cmap.endswith("_d"):
             pal = ["#333333"]
             pal.extend(color_palette(cmap.replace("_d", "_r"), 2))
             cmap = blend_palette(pal, as_cmap=True)
+    kwargs["cmap"] = cmap
     contour_func = ax.contourf if filled else ax.contour
-    contour_func(xx, yy, z, n_levels, cmap=cmap, **kwargs)
+    contour_func(xx, yy, z, n_levels, **kwargs)
+    kwargs["n_levels"] = n_levels
 
     # Label the axes
     if hasattr(x, "name") and axlabel:
@@ -616,15 +659,14 @@ def _bivariate_kdeplot(x, y, filled, kernel, bw, gridsize, cut, clip, axlabel,
 
 def _statsmodels_bivariate_kde(x, y, bw, gridsize, cut, clip):
     """Compute a bivariate kde using statsmodels."""
-    from statsmodels import nonparametric
     if isinstance(bw, str):
-        bw_func = getattr(nonparametric.bandwidths, "bw_" + bw)
+        bw_func = getattr(sm.nonparametric.bandwidths, "bw_" + bw)
         x_bw = bw_func(x)
         y_bw = bw_func(y)
         bw = [x_bw, y_bw]
     elif np.isscalar(bw):
         bw = [bw, bw]
-    kde = nonparametric.kernel_density.KDEMultivariate([x, y], "cc", bw)
+    kde = sm.nonparametric.KDEMultivariate([x, y], "cc", bw)
     x_support = _kde_support(x, kde.bw[0], gridsize, cut, clip[0])
     y_support = _kde_support(y, kde.bw[1], gridsize, cut, clip[1])
     xx, yy = np.meshgrid(x_support, y_support)
@@ -636,11 +678,13 @@ def _scipy_bivariate_kde(x, y, bw, gridsize, cut, clip):
     """Compute a bivariate kde using scipy."""
     data = np.c_[x, y]
     kde = stats.gaussian_kde(data.T)
+    data_std = data.std(axis=0, ddof=1)
     if isinstance(bw, str):
         bw = "scotts" if bw == "scott" else bw
-        bw = getattr(kde, "%s_factor" % bw)()
-    x_support = _kde_support(data[:, 0], bw, gridsize, cut, clip[0])
-    y_support = _kde_support(data[:, 1], bw, gridsize, cut, clip[1])
+        bw_x = getattr(kde, "%s_factor" % bw)() * data_std[0]
+        bw_y = getattr(kde, "%s_factor" % bw)() * data_std[1]
+    x_support = _kde_support(data[:, 0], bw_x, gridsize, cut, clip[0])
+    y_support = _kde_support(data[:, 1], bw_y, gridsize, cut, clip[1])
     xx, yy = np.meshgrid(x_support, y_support)
     z = kde([xx.ravel(), yy.ravel()]).reshape(xx.shape)
     return xx, yy, z
@@ -648,7 +692,7 @@ def _scipy_bivariate_kde(x, y, bw, gridsize, cut, clip):
 
 def kdeplot(data, data2=None, shade=False, vertical=False, kernel="gau",
             bw="scott", gridsize=100, cut=3, clip=None, legend=True, ax=None,
-            **kwargs):
+            cumulative=False, **kwargs):
     """Fit and plot a univariate or bivarate kernel density estimate.
 
     Parameters
@@ -681,6 +725,8 @@ def kdeplot(data, data2=None, shade=False, vertical=False, kernel="gau",
         If True, add a legend or label the axes when possible.
     ax : matplotlib axis, optional
         Axis to plot on, otherwise uses current axis.
+    cumulative : bool
+        If draw, draw the cumulative distribution estimated by the kde.
     kwargs : other keyword arguments for plot()
 
     Returns
@@ -692,25 +738,33 @@ def kdeplot(data, data2=None, shade=False, vertical=False, kernel="gau",
     if ax is None:
         ax = plt.gca()
 
+    data = data.astype(np.float64, copy=False)
+    if data2 is not None:
+        data2 = data2.astype(np.float64, copy=False)
+
     bivariate = False
     if isinstance(data, np.ndarray) and np.ndim(data) > 1:
         bivariate = True
-        x, y = data.astype(np.float64).T
+        x, y = data.T
     elif isinstance(data, pd.DataFrame) and np.ndim(data) > 1:
         bivariate = True
-        x = data.iloc[:, 0].values.astype(np.float64)
-        y = data.iloc[:, 1].values.astype(np.float64)
+        x = data.iloc[:, 0].values
+        y = data.iloc[:, 1].values
     elif data2 is not None:
         bivariate = True
         x = data
         y = data2
 
+    if bivariate and cumulative:
+        raise TypeError("Cumulative distribution plots are not"
+                        "supported for bivariate distributions.")
     if bivariate:
         ax = _bivariate_kdeplot(x, y, shade, kernel, bw, gridsize,
                                 cut, clip, legend, ax, **kwargs)
     else:
         ax = _univariate_kdeplot(data, shade, vertical, kernel, bw,
-                                 gridsize, cut, clip, legend, ax, **kwargs)
+                                 gridsize, cut, clip, legend, ax,
+                                 cumulative=cumulative, **kwargs)
 
     return ax
 
@@ -738,6 +792,10 @@ def rugplot(a, height=None, axis="x", ax=None, **kwargs):
     """
     if ax is None:
         ax = plt.gca()
+    a = np.asarray(a)
+    vertical = kwargs.pop("vertical", None)
+    if vertical is not None:
+        axis = "y" if vertical else "x"
     other_axis = dict(x="y", y="x")[axis]
     min, max = getattr(ax, "get_%slim" % other_axis)()
     if height is None:
@@ -748,3 +806,110 @@ def rugplot(a, height=None, axis="x", ax=None, **kwargs):
     else:
         ax.plot([min, min + height], [a, a], **kwargs)
     return ax
+
+
+def jointplot(x, y, data=None, kind="scatter", stat_func=stats.pearsonr,
+              color=None, size=6, ratio=5, space=.2,
+              dropna=True, xlim=None, ylim=None,
+              joint_kws=None, marginal_kws=None, annot_kws=None):
+    """Draw a plot of two variables with bivariate and univariate graphs.
+
+    Parameters
+    ----------
+    x, y : strings or vectors
+        Data or names of variables in `data`.
+    data : DataFrame, optional
+        DataFrame when `x` and `y` are variable names.
+    kind : { "scatter" | "reg" | "resid" | "kde" | "hex" }, optional
+        Kind of plot to draw.
+    stat_func : callable or None
+        Function used to calculate a statistic about the relationship and
+        annotate the plot. Should map `x` and `y` either to a single value
+        or to a (value, p) tuple. Set to ``None`` if you don't want to
+        annotate the plot.
+    color : matplotlib color, optional
+        Color used for the plot elements.
+    size : numeric, optional
+        Size of the figure (it will be square).
+    ratio : numeric, optional
+        Ratio of joint axes size to marginal axes height.
+    space : numeric, optional
+        Space between the joint and marginal axes
+    dropna : bool, optional
+        If True, remove observations that are missing from `x` and `y`.
+    {x, y}lim : two-tuples, optional
+        Axis limits to set before plotting.
+    {joint, marginal, annot}_kws : dicts
+        Additional keyword arguments for the plot components.
+
+    Returns
+    -------
+    grid : JointGrid
+        JointGrid object with the plot on it.
+
+    See Also
+    --------
+    JointGrid : The Grid class used for drawing this plot. Use it directly if
+                you need more flexibility.
+
+    """
+    # Set up empty default kwarg dicts
+    if joint_kws is None:
+        joint_kws = {}
+    if marginal_kws is None:
+        marginal_kws = {}
+    if annot_kws is None:
+        annot_kws = {}
+
+    # Make a colormap based off the plot color
+    if color is None:
+        color = color_palette()[0]
+    color_rgb = mpl.colors.colorConverter.to_rgb(color)
+    colors = [set_hls_values(color_rgb, l=l) for l in np.linspace(1, 0, 12)]
+    cmap = blend_palette(colors, as_cmap=True)
+
+    # Initialize the JointGrid object
+    grid = JointGrid(x, y, data, dropna=dropna,
+                     size=size, ratio=ratio, space=space,
+                     xlim=xlim, ylim=ylim)
+
+    # Plot the data using the grid
+    if kind == "scatter":
+
+        grid.plot_joint(plt.scatter, color=color, **joint_kws)
+        grid.plot_marginals(distplot, kde=False, color=color, **marginal_kws)
+
+    elif kind.startswith("hex"):
+
+        x_bins = _freedman_diaconis_bins(grid.x)
+        y_bins = _freedman_diaconis_bins(grid.y)
+        gridsize = int(np.mean([x_bins, y_bins]))
+
+        grid.plot_joint(plt.hexbin, gridsize=gridsize, cmap=cmap, **joint_kws)
+        grid.plot_marginals(distplot, kde=False, color=color, **marginal_kws)
+
+    elif kind.startswith("kde"):
+
+        grid.plot_joint(kdeplot, shade=True, cmap=cmap, **joint_kws)
+        grid.plot_marginals(kdeplot, shade=True, color=color, **marginal_kws)
+
+    elif kind.startswith("reg"):
+
+        from .linearmodels import regplot
+        grid.plot_marginals(distplot, color=color, **marginal_kws)
+        grid.plot_joint(regplot, color=color, **joint_kws)
+
+    elif kind.startswith("resid"):
+
+        from .linearmodels import residplot
+        grid.plot_joint(residplot, color=color, **joint_kws)
+        x, y = grid.ax_joint.collections[0].get_offsets().T
+        distplot(x, color=color, kde=False, ax=grid.ax_marg_x)
+        distplot(y, color=color, kde=False, vertical=True,
+                 fit=stats.norm, ax=grid.ax_marg_y)
+        stat_func = None
+
+    if stat_func is not None:
+        grid.annotate(stat_func, **annot_kws)
+
+    return grid
